"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _moment = _interopRequireDefault(require("moment"));

var _ajv = _interopRequireDefault(require("ajv"));

var _file = _interopRequireDefault(require("formidable/lib/file"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

const converts = {
  'date': v => _moment.default.utc(v, 'YYYY-MM-DD').toDate(),
  'time': v => _moment.default.utc(v, 'HH:mm:ssZ.SSS').toDate(),
  'date-time': v => _moment.default.utc(v).toDate()
};

class Validator {
  constructor() {
    this.ajv = new _ajv.default({
      coerceTypes: true,
      useDefaults: true
    });
    this.ajv.addKeyword('convert', {
      compile(convert, schema) {
        const fn = convert === true ? converts[schema.format] : convert;

        if (fn && typeof fn === 'function') {
          return (value, fullKey, data, key) => {
            data[key] = fn(value);
            return true;
          };
        }

        return () => true;
      }

    });
    this.ajv.addKeyword('file', {
      compile(checkFile, schema) {
        if (checkFile) {
          return value => {
            if (value && value instanceof _file.default) {
              return true;
            }

            return false;
          };
        }

        return () => true;
      }

    });
    this.schemas = {};
  }

  getSchemas() {
    return this.schemas;
  }

  addSchema(schemaName, props, options = {}) {
    (0, _utils.assert)(schemaName, 'schemaName is required');

    const _transformExtends = (0, _utils.transformExtends)(schemaName),
          name = _transformExtends.name,
          parents = _transformExtends.parents;

    let schema = null;

    if (props && Object.keys(props).length) {
      const properties = {};
      const requiredArr = options.required || [];
      Object.keys(props).forEach(k => {
        const _props$k = props[k],
              type = _props$k.type,
              required = _props$k.required,
              others = _objectWithoutProperties(_props$k, ["type", "required"]);

        const typeObj = (0, _utils.transformType)(type);
        properties[k] = _objectSpread({}, typeObj, others);

        if (required) {
          requiredArr.push(k);
        }
      });
      const required = [...new Set(requiredArr)];
      schema = {
        type: 'object',
        properties,
        required
      };
    }

    if (parents.length) {
      if (schema) {
        parents.push(schema);
      }

      schema = _objectSpread({}, options, {
        allOf: parents
      });
    } else {
      schema = _objectSpread({}, options, schema);
    }

    this.schemas[name] = schema;
    this.ajv.addSchema(schema, (0, _utils.ref)(name));
  }

  compile(schema) {
    return this.ajv.compile(schema);
  }

}

exports.default = Validator;